/*
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/

// This example demonstrates most features of the BeastLink C++ API.

#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <numeric>
#include <sstream>

#include "beastlink++.h"

using namespace std;
using namespace beastlink;

namespace {

// Convert "value" to hex string using "digits" digits.
template<typename T>
string to_hex(const T value, int digits) {
	stringstream ss;
	ss << hex << setw(digits) << setfill('0') << value;
	return ss.str();
}

} // namespace

// We use VID and PID for EFM-03 devices.
const uint16_t VID = 0x10f8;
const uint16_t PID = 0xc583;

// Do a device/FPGA design example specific test for each
// device that ships with an example design.
void deviceTest() {
	// Iterate over the list of detected devices.
	for (auto &dev : enumerate(VID, PID)) {
		// Open the device.
		auto d = dev.open();

		// This is the example design by Cesys, the design is read from
		// the specified file and programmed to the FPGA on the device (must be .bin format).
		// Optionally, programFpgaFromMemory() can
		// be used to download a compiled in resource (generated by tools).
		// resetFpga() is explicitly invoked from within this call.
		d.programFpgaFromBin("./beastlink_exdes_top.bin");

		// The example maps 16K Block RAM onto the AXI-4 Bus, starting at address 0x00000000.
		const auto BlockRAMAddr = uint32_t(0x00000000);
		const auto BlockRAMSize = 16 * 1024;
		const auto HalfRAMSize = BlockRAMSize / 2;

		// Create 16kb block containing counting 32 bit values.
		vector<uint8_t> data(BlockRAMSize);
		iota(
			reinterpret_cast<int*>(data.data()),
			reinterpret_cast<int*>(data.data() + data.size()),
			0
		);

		// Write to Block RAM.
		d.writeBlock(BlockRAMAddr, data);

		// Read back, starting at 8k.
		const auto rbdata = d.readBlock(BlockRAMAddr + HalfRAMSize, HalfRAMSize);

		// Compare second half of written data to read back data.
		const vector<uint8_t> shalf(data.begin() + HalfRAMSize, data.end());

		// Write compare result.
		cout
			<< "Sequences are "
			<< (equal(rbdata.begin(), rbdata.end(), shalf.begin()) ? "equal" : "not equal")
			<< ".\n"
			;
	}
}

// Enumerate all known devices and write information to stdout.
void infoTest() {
	// Loop over all devices.
	for (auto &dev : enumerate(VID, PID)) {
		// EnumeratedDevice::open() returns the opened instance to
		// the device. The device is implicitly closed if d goes out of scope.
		auto d = dev.open();

		// This structure contains all information about the device.
		// Not all devices have identical information. Devices, which
		// don't have a property return a default value, no error is thrown.
		const auto di = d.getDeviceInfo();

		// Write all informations to stdout, values are self explaining.
		cout
			<< "Device found: " << endl
			<< "  VID             : " << to_hex(di.getVid(), 4) << endl
			<< "  PID             : " << to_hex(di.getPid(), 4) << endl
			<< "  UserId          : " << di.getUserId() << endl
			<< "  SerialNumber    : " << di.getSerialNumber() << endl
			<< "  FirmwareVersion : " << di.getFirmwareVersion().toString() << endl
			<< "  DerivateInfo    : " << di.getDerivateInfo() << endl
			<< "  DerivateId      : 0x" << to_hex(di.getDerivateId(), 8) << endl
			;
	}
}

// Application entry point.
int main() {
	// Any error is reported by std::exception, catch it globally.
	try {
		// Initialize API before using it.
		init();

		// Output version to stdout.
		cout << "BeastLink version: " + getVersion().toString() << endl;

		// Set log level to log up to warning level (default).
		// Valid log levels are:
		// - 0x00: Off
		// - 0x01: Error
		// - 0x02: Warning
		// - 0x03: Debug
		setLogLevel(0x2);

		// Write all informations about all known connected
		// devices to stdout.
		infoTest();

		// Do a device specific test (with devices that have an
		// example FPGA design).
		deviceTest();

		// Cleanup internal structures. This closes all
		// opened resources, no further use of the UDK3 API
		// is possible until init() is called again.
		cleanup();
	} catch (exception &ex) {
		// In case of an error, write reason to stdout and the application stops.
		cout << endl << endl << "Exception: " << ex.what() << endl;
	}

	return 0;
}
