﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/

using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace BeastLink {
    /// <summary>
    /// Class that is responsible for all device independent functionality.
    /// </summary>
    public sealed class LibraryInterface
    {
        internal static int _enumRound = 0;

        /// <summary>
        /// Return full version info of BeastLink.
        /// </summary>
        public static Version Version {
            get {
                int major = 0, minor = 0, fix = 0;
                SystemConnector.BlGetVersion(ref major, ref minor, ref fix);
                return new Version(major, minor, fix);
            }
        }

        /// <summary>
        /// The API is not usable before this function.
        /// </summary>
        public static void Init()
        {
            SystemConnector.BlInit();
        }

        /// <summary>
        /// Search for devices with given Vendor ID and Product ID.
        /// </summary>
        /// <param name="vid"></param>
        /// <param name="pid"></param>
        /// <returns>List of devices found in system and not already in use.</returns>
        public static IList<EnumeratedDevice> Enumerate(int vid, int pid)
        {
            ++_enumRound;
            return Enumerable
                .Range(0, SystemConnector.BlEnumerate(vid, pid))
                .Select(x => new EnumeratedDevice(new DeviceInfo(vid, pid), x, _enumRound))
                .ToList();
        }

        /// <summary>
        /// Free API resources, closes all devices. To reuse the API again,
        /// Init() must be called again.
        /// </summary>
        public static void Cleanup()
        {
            try
            {
                SystemConnector.BlCleanup();
            }
            catch (IOException) { }
        }

        /// <summary>
        /// Set logging level (Off = 0, Error = 1, Warning = 2, Debug = 3), default is 2.
        /// </summary>
        /// <param name="level"></param>
        public static void SetLogLevel(int level)
        {
            SystemConnector.BlSetLogLevel(level);
        }
    }
}
