/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2018
*/  

package com.cesys.beastlink;

import java.io.IOException;
import java.util.Arrays;

import com.sun.jna.Native;
import com.sun.jna.Platform;

/**
 * Direct access to the Core API using JNA. Internal use only.
 */
class SystemConnector {
	static {
		Native.register(getLibraryName());
	}

	private SystemConnector() {
	}

	private static String getLibraryName() {
		final String version = "1.0";
		final String prefix = "beastlink";
		if (Platform.isWindows()) {
			if (System.getProperty("os.arch").contains("64"))
				return String.format("%s-%s-x86_64", prefix, version);
			else
				return String.format("%s-%s-x86", prefix, version);
		}
		return String.format("%s-%s", prefix, version);
	}

	public static final int BL_SUCCESS = 0;
	public static final int BL_FAILED = -1;

	public static final int BL_INVALID_DEVICE = -1;

	public static final int BL_IF_BUFFER_SIZE = 16384;

	public static final int BL_FLAG_CONST_ADDR = 0x00000001;   // Use const address in block I/O (BlReadBlock/BlWriteBlock).

	static void checkError(int errorCode) throws IOException {
		if (errorCode != BL_SUCCESS) {
			byte[] error = new byte[SystemConnector.BL_IF_BUFFER_SIZE];
			BlGetLastErrorText(errorCode, error);
			throw new IOException(convertByteArrayToString(error));
		}
	}

	/**
	 * Extract String from byte[] while taking String terminator into account.
	 * @param array
	 * @return
	 */
	static String convertByteArrayToString(byte[] array) {
		// Find string terminator in path.
		int pos = 0;
		for (int l = 0; l < array.length; ++l, ++pos)
			if (0 == array[l])
				break;
		byte[] cutted = Arrays.copyOf(array, pos);
		return new String(cutted);
	}

	/**
	 * As Java has no uint32, the parameter must be a long, but it must result
	 * in an int, that holds the binary version of the uint32. Additional,
	 * an range check is done.
	 * @param value
	 * @return
	 */
	static int toUint32(long value) {
		final long maxValue = 4294967295L;
		final long minValue = 0;
		if (value < minValue || value > maxValue)
			throw new IllegalArgumentException("Value outside range of 32 bit unsigned int.");
		return (int)value;
	}

	/**
	 * Convert int to 32 bit unsigned representation in a long.
	 * @param value
	 * @return
	 */
	static long fromUInt32(int value) {
		if (value >= 0)
			return (long)value;
		return 0x100000000L + (long)value;
	}

	public static native void BlGetLastErrorText(int id, byte[] error);

	public static native void BlGetVersion(int[] major, int[] minor, int[] fix);

	public static native int BlInit();
	public static native int BlEnumerate(int vid, int pid, int[] count);
	public static native int BlCleanup();
	public static native int BlSetLogLevel(int level);

	public static native int BlOpen(int enumId, int[] handle);
	public static native int BlClose(int handle);

	public static native int BlReadRegister(int handle, int reg, int[] result);
	public static native int BlWriteRegister(int handle, int reg, int value);
	public static native int BlReadBlock(int handle, int addr, byte[] buffer, int size, int flags);
	public static native int BlWriteBlock(int handle, int addr, byte[] buffer, int size, int flags);

	public static native int BlResetFpga(int handle);
	public static native int BlProgramFpgaFromBin(int handle, String fileName);
	public static native int BlProgramFpgaFromMemory(int handle, byte[] buffer, int size, int compressed);

	public static native int BlSetTimeout(int handle, int timeout);

	public static native int BlSetUserId(int handle, byte[] userId);
	public static native int BlGetUserId(int handle, byte[] buffer);
	public static native int BlGetSerialNumber(int handle, byte[] buffer);
	public static native int BlGetFirmwareVersion(int handle, int[] major, int[] minor, int[] fix);

	public static native int BlGetDerivateInfo(int handle, byte[] buffer);
	public static native int BlGetDerivateId(int handle, int[] derivateId);
}
